using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Serialization;
using System.Threading;

using Sturm.Util.ApplicationServices.ObjectPooling;

namespace ObjectPoolingSample {
	class Program : IObjectFactory<PoolableObject> {
		static void Main(string[] args) {
			new Program( ).Run( );
		}
		
		Pool<PoolableObject> pool;
		private const int threadCount = 3;

		void Run( ) {
			pool = new Pool<PoolableObject>(this, 0);
			pool.InitializeCounters( );
			pool.UseTimerBasedResizing = true;
			pool.MaxPoolSize = 100;

			Thread[] threads = new Thread[threadCount];
			for (int i = 0; i < threadCount; i++) {
				threads[i] = new Thread(new ThreadStart(ThreadMethod));
				threads[i].Start( );
			}

			while (true) {
				Console.WriteLine("Current pool size: " + pool.Size);
				Thread.Sleep(2000);
			}
		}

		void ThreadMethod( ) {
			Random random = new Random( );
			while (true) {
				if (random.Next(0, 100) > 50) {
					// boost
					int boostSize = random.Next(5, 20);
					PoolableObject[] objects = new PoolableObject[boostSize];
					for (int i = 0; i < boostSize; i++) {
						objects[i] = pool.GetObject( );
						Console.WriteLine(String.Format("Thread {0} allocated object {1}.", Thread.CurrentThread.ManagedThreadId, objects[i].Id));
						objects[i].DoWork( );
					}
					Thread.Sleep(random.Next(0, 5000));
					for (int i = 0; i < boostSize; i++) {
						pool.ReleaseObject(objects[i]);
						Console.WriteLine(String.Format("Thread {0} released object {1}.", Thread.CurrentThread.ManagedThreadId, objects[i].Id));
					}
				}
				else {
					// single
					PoolableObject poolableObject = pool.GetObject( );
					Console.WriteLine(String.Format("Thread {0} allocated object {1}.", Thread.CurrentThread.ManagedThreadId, poolableObject.Id));
					poolableObject.DoWork( );
					Thread.Sleep(random.Next(0, 5000));
					pool.ReleaseObject(poolableObject);
					Console.WriteLine(String.Format("Thread {0} released object {1}.", Thread.CurrentThread.ManagedThreadId, poolableObject.Id));
				}
			}
		}

		#region IObjectFactory<PoolableObject> Members

		PoolableObject IObjectFactory<PoolableObject>.CreateObject( ) {
			return new PoolableObject( );
		}

		#endregion
	}

	public class PoolableObject {
		public PoolableObject( ) {
			id = Guid.NewGuid( );
			Console.WriteLine(String.Format("Poolable object {0} is being constructed.", id));
		}

		~PoolableObject( ) {
			Console.WriteLine(String.Format("Poolable object {0} is being destructed.", id));
		}

		private Guid id;
		public Guid Id {
			get {
				return id;
			}
		}

		int useCount;

		public void DoWork( ) {
			// Obviously this object should be able to actually do something, but
			// that's not relevant to the test program.
			// So we just count how many times an object has been used.
			Console.WriteLine(String.Format("Poolable object {0} has been used {1} times now.", id, ++useCount));
		}
	}
}
